///<section name="is.xxx library">
/// nakolko javascript je weakly typed language, pomerne casto je problem identifikovat,
/// co za predmet (parameter, premenna) mam vlastne v rukach.
/// Aby sa nemuselo zdlhavo v kode vypisovat ' if (typeof VEC == 'something')
/// hodi sa sada funkcii umoznujuca skratkovito detekovat, co je vec vlastne zac
/*# is library definicia je zabalena podla
VZORu moznosti univerzalnej definicie kontext-u (browser...window, nodejs...global)
a teda funkcna aj v browser-i, aj pre nodejs server-side funkcionalitu
*/
(function(glob){	//! is.xxx library
//http://ejohn.org/blog/ecmascript-5-strict-mode-json-and-more/
// Non-strict code...
  "use strict";
// Define your library strictly...

///<var name="__" type="undefined" access="global"> alias for undefined
//# Moznost pouzivat v kode '__' miesto 'undefined' sa mi paci omnoho viac
//# preto vyrobim globalnu premennu '__'
  glob.__ = undefined; // __ double undescore like a undefined
///</var>

///<function name="getType" access="global"> 
//# funkcia getType vola javascript_core funkciu na urcenie 'pisaneho' typu 
//# produkujucou napr. getType(window) ... "[object global]"
	function getType(thing){
		if(thing===null)return "[object Null]"; // special case
		return Object.prototype.toString.call(thing);
	}
///</function>
///<function name="getObjectType"> 
//# funkcia getObjectType vyhryzne z getType odpovede to podstatne
//# napr. iS._objType(window) ... "global" 
	function getObjectType(thing){
		var oT = getType(thing);
		var matches = /\[object\s+(.+)\]/.exec(oT);
		if (matches && matches[1]) {
			return matches[1];
		}
		return null;
	}
///</function>
///<object name="is">
/*# identifikacia druhu javascript objektov nie je uplne jednoducha,
ale pouzity kod by mal byt uz dostatocne otestovany.

Dolezita poznamka : pomerne casto je potrebne v kode vediet, ci nieco uz bolo definovane
ci uz v globalnom kontexte (univarzalne pristupne odkialkolvek),
alebo v closure (v kontextoch hierarchicky nadbalane nad kusom kodu, v ktorom sa nachadzam
- v nodejs aj kazdy volany modul predstavuje closure, standartne kazda funkcia vytvara
closure, takze funkcia definovana vnutri funkcie pozna aj premenne funkcie, do ktorej
je vnorena ako closure premenne).
	Testovanie pomocou funkcie isDef (is._def) je mozne len na parametre funkcii
(ktore tiez moznu byt zadane ako undefined), nakolko taketo parametre existuju ako
javascriptove veci s hodnotou undefined.
	Ked vsak ide o zistovanie, ci existuje globalna premenna , ci closure premenna,
javascript vyhlasi chybu pri pokuse vlozit meno nedefinovanej premennej do parametra
funkcie isDef ... a preto neostava nic ineho, len konstrukcia
	if (typeof TOTO == 'undefined') ....
Aj tak funkcia isDef statocne pomaha pri identifikacii parametrov s hodnotou undefined
*/
  var is = {	
	 _type : getType	//access direct to getType
	,_objType : getObjectType	//access direct to getType
  
	,_null : function(thing) {	return thing === null; }
	,_false: function(thing) {	return thing === false; }
	,_true : function(thing) {	return thing === true; }
//	,_empty: function(thing) {	return (	( thing instanceof Object )
//										? 	(Object.getOwnPropertyNames(thing).length == 0) 
//										: (! thing)
//										);
//							}
	,_empty : function(thing) {
		var op = ( thing instanceof Object ) ? Object.getOwnPropertyNames(thing) : null;
		return (	op
				? 	(   (op.length == 0) 
					 || ( ( thing instanceof Array ) 
							&& (op.length == 1)
							&&( op[0] == "length" )   )
					)
				: (! thing)
				);
	}
	,_def: function(thing) { return (thing !== undefined); }
	,_undef: function(thing) { return (thing === undefined); }
	,_none: function(thing) { return (thing === undefined) || (thing === null); }
	/* test
	x = undefined; y=false; z=""; w=null; q="q";
	f=isSet;alert('func:isSet x(undefined):'+f(x)+' y(false):'+f(y)+' z(""):'+f(z)+' w(null):'+f(w)+' q("q"):'+f(q));
	f=isFalse;alert('func:isFalse x(undefined):'+f(x)+' y(false):'+f(y)+' z(""):'+f(z)+' w(null):'+f(w)+' q("q"):'+f(q));
	f=isTrue;alert('func:isTrue x(undefined):'+f(x)+' y(false):'+f(y)+' z(""):'+f(z)+' w(null):'+f(w)+' q("q"):'+f(q));
	f=isEmpty;alert('func:isEmpty x(undefined):'+f(x)+' y(false):'+f(y)+' z(""):'+f(z)+' w(null):'+f(w)+' q("q"):'+f(q));
	*/

	,_object  : function(thing) { return getType(thing) == "[object Object]"; }
	,_bool    : function(thing) { return getType(thing) == "[object Boolean]"; }
	,_number  : function(thing) { return getType(thing) == "[object Number]"; }
// `NaN` is the only value for which `===` is not reflexive.
	,_isNaN   : function(obj) { return obj !== obj; }
	,_string  : function(thing) { return getType(thing) == "[object String]"; }
	,_array   : function(thing) { return Array.isArray ? Array.isArray(thing) : getType(thing) == "[object Array]"; }
	,_function: function(thing) { return getType(thing) == "[object Function]"; }

	,_node: function(thing) { return thing instanceof Node; }
	,_element: function(thing) { return thing instanceof Element; }
	,_svg: function(thing) { return thing instanceof SVGElement; }
	,_html: function(thing) { return thing instanceof HTMLElement; }
	,_xml: function(thing) { return thing instanceof Element && !(is._html(thing)) && !(is._svg(thing)); }	//????
	
	,_inDOM	: function(el) {
			if (!isNOD(el)) {
				return false;
			}
			do {
				if (el instanceof Document) {
					return true;
				}
				el = el.parentNode;
			} while(el);
			return false;
		}
  }
///</object>
///<object name="iS" access="global"> globalized object <ref>is</ref>
//# kniznicka is je uvedena do globalneho kontextu ako iS 
//# kvoli existencii inych kniznic, ktore pojem 'is' pouzivaju
  glob.iS = is;
///</object>

///<section name="is_aliases"> direct global access to is_XXX functions
/*# definicny zoznam is_XXX skratiek pristupnych kedykolvek a kdekolvek
	isDef 	: is._def
	isNul 	: is._null
	isFal 	: is._false
	isTru 	: is._true
	isEmp 	: is._empty
	isNon	: is._none
	isObj 	: is._object
	isBoo 	: is._bool
	isNum 	: is._number
	isNan 	: is._isNaN
	isStr 	: is._string
	isArr 	: is._array
	isFnc 	: is._function

	isNOD 	: is._node
	isELE 	: is._element
	isSVG 	: is._svg
	isHTM 	: is._html
	isXML 	: is._xml
	
	isInDOM : is._inDOM
*/
  var aliases = {  
		isTyp 	: is._type
	,	getType : is._type
	,	objType : is._objType
		//-------------- my favourites:
	,	isDef 	: is._def
	,	isNul 	: is._null
	,	isFal 	: is._false
	,	isTru 	: is._true
	,	isEmp 	: is._empty
	,	isNon	: is._none
	,	isObj 	: is._object
	,	isBoo 	: is._bool
	,	isNum 	: is._number
	,	isNan 	: is._isNaN
	,	isNbr 	: function(thing) { return is._number(thing) && ! (is._isNaN(thing)); } 
	,	isStr 	: is._string
	,	isArr 	: is._array
	,	isFnc 	: is._function

	,	isNOD 	: is._node
	,	isELE 	: is._element
	,	isSVG 	: is._svg
	,	isHTM 	: is._html
	,	isXML 	: is._xml
	
	,	isInDOM : is._inDOM
/*
	//-------------- more verbose: // uncomment if unconditionally needed - not recommended due to compatibility issuses
	,	isSet 		: is._def
	,	isDefined 	: is._def
	,	isNull 		: is._null
	,	isFalse 	: is._false;
	,	isTrue 		: is._true
	,	isEmpty 	: is._empty
	,	isNone		: is._none
	,	isObject 	: is._object
	,	isBool 		: is._bool
	,	isNumber 	: is._number
	,	isString 	: is._string
	,	isArray 	: is._array
	,	isFunction 	: is._function

	,	isSVG 		: is._svg
	,	isHTML	 	: is._html
	,	isXML 		: is._xml
*/
	};
	
//# a zavedenie skratkovitych funkcii do globalneho kontextu
// extend glob if You like :
  for (var item in aliases) {
	glob[item] = aliases[item];
  }
///</section>
//# a uzavierame obal kniznicky is 
//# rozlisenim podla VZOR-u moznosti univerzalnej definicie kontext-u 
//# ci ide o browser...window, alebo nodejs...global  
})(
	(typeof window != "undefined") ? window : ((typeof global != "undefined") ? global : this)
   );

// Non-strict code... 
///</section>

//	gopn(Actual_CalculForms.funcs,'funcs',__,  function(o) {
//		if (o && (typeof o.ac != 'undefined')) {
//			o.ac = false;
//	    }
//		return o;
//	})
function gopn(o, pre, r, callback) {
  var a,c;
  r  = r || [];
  if (typeof callback == 'function') {
	o = callback(o, pre, r);
  }
  if ( ((typeof o) != "string") && ((typeof o) != "function") ) {
	  try {
	   a = Object.getOwnPropertyNames(o);
	  } catch(e) {}
  }
  if (a) {
    for (var i=0; i<a.length; i++) {
    	c=[];
  		if ( ((typeof o[a[i]]) != "string") && ((typeof o[a[i]]) != "function") ) {
			try {
				c = Object.getOwnPropertyNames(o[a[i]]);
			} catch(e){}
  		}
      if (!c.length) {	// neukladame mena objektov
		  r.push(pre+(pre?".":"")+a[i]);
      }
      r = gopn(o[a[i]], pre+(pre?".":"")+a[i], r, callback);
    }
  }
  return r;
}
